const megadb = require('megadb');
const mongoose = require('mongoose');

class model {
    constructor(dbType = String, Mongo = {
        uri: String,
        MongoOptions: Object
    }) {
        if(dbType != 'megadb' && dbType != 'maps' && dbType != 'mongoose') throw new Error('You must to type the db type. Options: megadb, maps, mongoose');
    
        if(dbType === 'megadb') {
            this.dbType = 'megadb';
        }else if(dbType === 'maps') {
            this.dbType = 'maps';
        }else if(dbType === 'mongoose') {
            this.dbType = 'mongoose';
            mongoose.connect(Mongo.uri, Mongo.MongoOptions ?? { useNewUrlParser: true, useUnifiedTopology: true });
        }
    }

    async createModel(name = String, modelSchema = Object) {
        if(!modelSchema.key) modelSchema.key = String;

        if(this.dbType === 'megadb') {
            this.modelDB = new megadb.crearDB(name ?? 'untitled');
            this.modelSchema = { name: name, modelSchema: modelSchema };
        }else if(this.dbType === 'maps') {
            this.modelDB = new Map();
            this.modelSchema = { name: name, modelSchema: modelSchema };
        }else if(this.dbType === 'mongoose') {
            this.modelDB = mongoose.model(name, new mongoose.Schema(modelSchema));
            this.modelSchema = { name: name, modelSchema: modelSchema };
        }
    }

    async set(key = String, schema = Object) {
        if(!key) key = '0';

        if(this.dbType === 'megadb') {
            if(!schema.key) schema.key = key;
            this.modelDB.set(key, schema);
        }else if(this.dbType === 'maps') {
            if(!schema.key) schema.key = key;
            this.modelDB.set(key, schema);
        }else if(this.dbType === 'mongoose') {
            if(!schema.key) schema.key = key;
            let newDoc = new this.modelDB(schema);
            await newDoc.save();
        }
    }

    async has(key = String) {
        if(!key) key = '0';

        if(this.dbType === 'megadb') {

            if(this.modelDB.has(key)) return true;
            else return false;

        }else if(this.dbType === 'maps') {

            if(this.modelDB.has(key)) return true;
            else return false;

        }else if(this.dbType === 'mongoose') {

            let Doc = await this.modelDB.findOne(key);
            if(Doc) return true;
            else return false;

        }
    }

    async delete(key = String) {
        if(!key) key = '0';

        if(this.dbType === 'megadb') {
            this.modelDB.delete(key);
        }else if(this.dbType === 'maps') {
            this.modelDB.delete(key);
        }else if(this.dbType === 'maps') {
            this.modelDB.delete(key);
        }else if(this.dbType === 'mongoose') {
            await this.modelDB.findOneAndDelete(key);
        }
    }

    async get(key = String) {
        if(!key) key = '0';

        if(this.dbType === 'megadb') {
            return await this.modelDB.get(key);
        }else if(this.dbType === 'maps') {
            return await this.modelDB.get(key);
        }else if(this.dbType === 'mongoose') {
            let Doc = await this.modelDB.findOne(key);
            return Doc;
        }
    }

    async getAll(filter = Object) {
        if(this.dbType === 'megadb') {
            return await this.modelDB.filter(false, doc => JSON.stringify(doc).includes(JSON.stringify(filter).slice(1, 1)));
        }else if(this.dbType === 'maps') {
            let arr = [];
            await this.modelDB.forEach(doc => { if(JSON.stringify(doc).includes(JSON.stringify(filter).slice(1, 1))) arr.push(doc); });
            return arr;
        }else if(this.dbType === 'mongoose') {
            return await this.modelDB.find(filter);
        }
    }

    async update(key = String, newSchema = Object) {
        if(!key) key = '0';

        if(this.dbType === 'megadb') {
            this.set(key, newSchema);
        }else if(this.dbType === 'maps') {
            this.delete(key);
            this.set(key, newSchema);
        }else if(this.dbType === 'mongoose') {
            await this.modelDB.findOneAndUpdate(key, newSchema);
        }
    }

}

module.exports = { model };